function Nint = sparsifyNint_OCTAVE(model)
% Sparsify Nint solving a Null-space Basis Problem
% Inputs:     model structure, params (solver parameters, optional)
%
% Outputs:    Nsnp (sparse null space matrix for internal loops)
%%%%%%%%%%%%%%%%%%%%%% Pedro Saa UQ 2016 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define solver parameters         
params.tolobj  = 1e-6;          
params.tolbnd  = 1e-6;
params.toldj   = 1e-6;
params.msglev  = 0;     % No messages

% Extract information only about internal reactions
model.S  = model.S(:,model.internal);
model.lb = -1e3*(model.lb(model.internal)<0);
model.ub = 1e3*(model.ub(model.internal)>0);

% Initialization of SNP parameters
nullity = size(null(full(model.S)),2);
numRxns = size(model.S,2);
P_N     = 0;
Nint    = [];
fluxTol = 1e-6;
epsilon = 1e-1;

%% Main loop
for jx = 1:nullity  % Iterate until we reached the desired tolerance
    Ntemp    = [];
    P_NT     = eye(numRxns)-P_N'*P_N;
    weights  = 1+.2*rand(1,size(model.S,2));
    
    % Solve Problem 1: w'*P*v > epsilon    
    modelSNP = nullSparseBasisStructure_GLPK(model,weights*P_NT);
    modelSNP.rhs(end)   = epsilon;
    modelSNP.ctype(end) = 'L';
    [xopt,~,status] = glpk(modelSNP.obj,modelSNP.A,modelSNP.rhs,modelSNP.lb,modelSNP.ub,modelSNP.ctype,modelSNP.vartype,modelSNP.sense,params);
    if (status==0)
       vopt = xopt(1:numRxns);  % Set to zero entries below tol
       vopt(abs(vopt)<fluxTol) = 0;
       min_val = min(abs(vopt(vopt~=0)));
       Ntemp   = [Ntemp,vopt/min_val];
    end
    
    % Solve Problem 2: w'*P*v < -epsilon
    modelSNP = nullSparseBasisStructure_GLPK(model,weights*P_NT);
    modelSNP.rhs(end)   = -epsilon;
    modelSNP.ctype(end) = 'U';
    [xopt,~,status] = glpk(modelSNP.obj,modelSNP.A,modelSNP.rhs,modelSNP.lb,modelSNP.ub,modelSNP.ctype,modelSNP.vartype,modelSNP.sense,params);
    if (status==0)
        vopt = xopt(1:numRxns);  % Set to zero entries below tol
        vopt(abs(vopt)<fluxTol) = 0;
        min_val = min(abs(vopt(vopt~=0)));
        Ntemp   = [Ntemp,vopt/min_val];
    end
    
    % Select best solution
    if isempty(Ntemp)
        disp(['Potentially active loop laws found: ',num2str(jx-1),'/',num2str(nullity)]);
        break;
    end
    [~,ix] = max(sum(Ntemp==0));       % Keep sparsest vector
    Nint = [Nint,Ntemp(:,ix)];
    
    % Re-build projetion onto col(x)
    P_N = orth(Nint)';
end